// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package main

import (
	"bytes"
	"encoding/base64"
	"encoding/hex"
	"fmt"
	"math/rand"
	"net/http"
	"testing"

	"hello-api/internal/logger"
	"hello-api/internal/utils"

	deviceauthpb "carbonblack.com/protocols/psc/ems/server/deviceauth"
	. "github.com/onsi/ginkgo/v2"
	. "github.com/onsi/gomega"
	"google.golang.org/protobuf/proto"
)

const (
	helloAPIRoute       = "services/hello"
	deviceContextHeader = "Device-Context"
)

func getEndpoint() string {
	return fmt.Sprintf("http://localhost:8080/%s", helloAPIRoute)
}

//goland:noinspection GoUnhandledErrorResult
var _ = Describe("functional tests", func() {
	deviceId := rand.Int63()
	const (
		orgKey             = "7ZP4H5XR" // ENV local, OrgId 2023
		orgId              = 2023
		deviceGroupId      = 456
		defaultStreamName  = "local-ems-device-hello-local-infra-lct-stream-psc-ems-last-contact-time-updater-stream"
		defaultAwsRegion   = "us-east-1"
		defaultAwsEndpoint = "http://localhost:4566"

		// Encoded:
		//  - version: 17
		//  - lastUpdateTime: Sunday, February 2, 2020 12:00:00 GMT, 1580644800000
		//  - lastLiveQueryTime: Monday, September 9, 2019 12:00:00 GMT, 1568030400000
		encodedHeaderString = "c011c0c0c0c0c0c0c0c0c0d30000017005c5960081b14c6173744c697665517565727954696d65cf0000016d15e51e00"
	)

	httpClient := http.DefaultClient

	testContextHeader := &deviceauthpb.Context{
		Device: &deviceauthpb.Device{
			Id: deviceId, DeviceType: deviceauthpb.DeviceType_LINUX,
			DeploymentType: deviceauthpb.DeploymentType_ENDPOINT, OrgKey: orgKey,
		},
		DeviceGroup:  &deviceauthpb.DeviceGroup{Id: deviceGroupId, OrgId: orgId},
		Organization: &deviceauthpb.Organization{OrgKey: orgKey, Id: orgId},
	}
	logger.BaseLogger().Debug(fmt.Sprintf("Device Context: %+v", testContextHeader))

	marshalledHeader, _ := proto.Marshal(testContextHeader)
	encodedHeader := base64.StdEncoding.EncodeToString(marshalledHeader)

	// NOTE: negative tests beforehand to ensure they don't bring down the service!

	It("sensors with dereg header set get deregister message back", func() {
		logger.BaseLogger().Info(utils.HeaderStr("Validate dereg"))
		logger.BaseLogger().Info(utils.HeaderStr("Validate hello API response"))
		decodedBuffer, _ := hex.DecodeString(encodedHeaderString)
		request, err := http.NewRequest("POST", getEndpoint(), bytes.NewBuffer(decodedBuffer))
		if err != nil {
			logger.BaseLogger().Error(fmt.Sprintf("Failed to get new request: %v", err))
		}
		Expect(err).To(BeNil())
		Expect(request).ToNot(BeNil())

		request.Header.Set(deviceContextHeader, encodedHeader)
		request.Header.Set("Device-Unauthorized", "1")
		logger.BaseLogger().Info(fmt.Sprintf("Calling httpClient Do() to %s", getEndpoint()))
		logger.BaseLogger().Info(fmt.Sprintf("Msgpack: %s", decodedBuffer))
		logger.BaseLogger().Info(fmt.Sprintf("Device Context: %v (%s)", testContextHeader, encodedHeader))
		response, err := httpClient.Do(request)
		if err != nil {
			logger.BaseLogger().Error(fmt.Sprintf("Failed to make POST call: %v", err))
		}
		Expect(err).To(BeNil())
		defer response.Body.Close()

		// Check 1: Hello Response Code
		Expect(response).ToNot(BeNil())
		Expect(response.StatusCode).To(BeComparableTo(503))
	})
})

func TestDereg(t *testing.T) {
	RegisterFailHandler(Fail)
	RunSpecs(t, "dereg tests")
}
